@@ -44,6 +44,7 @@
 #endif
 #include <ksqueezedtextlabel.h>
 
+#include <KCollapsibleGroupBox>
 // Some i18n filters, that standard button texts are piped through
 // (the new KGuiItem object with filtered text is created from the old one).
 
@@ -155,9 +156,6 @@ public Q_SLOTS:
         QDialogButtonBox::StandardButton code = m_buttons->standardButton(button);
         if (code != QDialogButtonBox::NoButton) {
             m_dialog->done(code);
-        } else if (m_details && (button->objectName() == QStringLiteral("detailsButton"))) {
-            button->setText(QApplication::translate("KMessageBox", "&Details") + (m_details->isVisible() ? QStringLiteral(" >>") : QStringLiteral(" <<")));
-            m_details->setVisible(!m_details->isVisible());
         }
     }
 
@@ -206,11 +204,11 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
     QVBoxLayout *mainLayout = new QVBoxLayout(mainWidget);
     const int spacingHint = mainWidget->style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
     mainLayout->setSpacing(spacingHint * 2); // provide extra spacing
-    mainLayout->setMargin(0);
+    mainLayout->setContentsMargins(0, 0, 0, 0);
     buttons->setParent(dialog);
 
     QHBoxLayout *hLayout = new QHBoxLayout();
-    hLayout->setMargin(0);
+    hLayout->setContentsMargins(0, 0, 0, 0);
     hLayout->setSpacing(-1); // use default spacing
     mainLayout->addLayout(hLayout, 5);
 
@@ -282,7 +280,7 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
         styleOption.initFrom(listWidget);
         QFontMetrics fm(styleOption.font);
         int w = listWidget->width();
-        Q_FOREACH (const QString &str, strlist) {
+        for (const QString &str : strlist) {
             w = qMax(w, fm.width(str));
         }
         const int borderWidth = listWidget->width() - listWidget->viewport()->width() + listWidget->verticalScrollBar()->height();
@@ -313,7 +311,10 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
     topLayout->addWidget(mainWidget);
 
     if (!details.isEmpty()) {
-        QGroupBox *detailsGroup = new QGroupBox(QApplication::translate("KMessageBox", "Details"));
+        QHBoxLayout *detailsHLayout = new QHBoxLayout();
+        detailsHLayout->addSpacing(2 * spacingHint + iconLayout->sizeHint().width());
+        KCollapsibleGroupBox *detailsGroup = new KCollapsibleGroupBox();
+        detailsGroup->setTitle(QApplication::translate("KMessageBox", "Details"));
         QVBoxLayout *detailsLayout = new QVBoxLayout(detailsGroup);
         if (details.length() < 512) {
             QLabel *detailsLabel = new QLabel(details);
@@ -324,6 +325,7 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
             };
             detailsLabel->setTextInteractionFlags(flags);
             detailsLabel->setWordWrap(true);
+            detailsLabel->setMinimumSize(detailsLabel->sizeHint());
             detailsLayout->addWidget(detailsLabel, 50);
         } else {
             QTextBrowser *detailTextEdit = new QTextBrowser();
@@ -335,21 +337,16 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
         if (!usingListWidget) {
             mainLayout->setStretchFactor(hLayout, 10);
         }
-        topLayout->addWidget(detailsGroup);
+        detailsHLayout->addWidget(detailsGroup);
+        topLayout->addLayout(detailsHLayout);
         buttonsHelper->setDetailsWidget(detailsGroup);
-        detailsGroup->hide();
     }
 
     topLayout->addWidget(buttons);
+    topLayout->setSizeConstraint(QLayout::SetFixedSize);
 
     if (!usingListWidget && !usingScrollArea && !usingSqueezedTextLabel && details.isEmpty()) {
         dialog->setFixedSize(dialog->sizeHint() + QSize(10, 10));
-    } else if (!details.isEmpty() && dialog->minimumHeight() < iconLabel->sizeHint().height() * 2) { //strange bug...
-        if (!usingScrollArea) {
-            dialog->setMinimumSize(300, qMax(150, qMax(iconLabel->sizeHint().height(), messageLabel->sizeHint().height())));
-        } else {
-            dialog->setMinimumSize(300, qMax(150, iconLabel->sizeHint().height()));
-        }
     }
 
     if ((options & KMessageBox::Dangerous)) {
@@ -809,13 +806,7 @@ static void detailedErrorInternal(QDialog *dialog, const QString &text,
     dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Error") : caption);
     dialog->setObjectName(QStringLiteral("error"));
 
-    QPushButton *detailsButton = new QPushButton;
-    detailsButton->setObjectName(QStringLiteral("detailsButton"));
-    detailsButton->setText(QApplication::translate("KMessageBox", "&Details") + QStringLiteral(" >>"));
-    detailsButton->setIcon(QIcon::fromTheme(QStringLiteral("help-about")));
-
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
-    buttonBox->addButton(detailsButton, QDialogButtonBox::HelpRole);
     buttonBox->addButton(QDialogButtonBox::Ok);
     buttonBox->button(QDialogButtonBox::Ok)->setFocus();
 
@@ -858,13 +849,7 @@ static void detailedSorryInternal(QDialog *dialog, const QString &text,
     dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Sorry") : caption);
     dialog->setObjectName(QStringLiteral("sorry"));
 
-    QPushButton *detailsButton = new QPushButton;
-    detailsButton->setObjectName(QStringLiteral("detailsButton"));
-    detailsButton->setText(QApplication::translate("KMessageBox", "&Details") + QStringLiteral(" >>"));
-    detailsButton->setIcon(QIcon::fromTheme(QStringLiteral("help-about")));
-
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
-    buttonBox->addButton(detailsButton, QDialogButtonBox::HelpRole);
     buttonBox->addButton(QDialogButtonBox::Ok);
     buttonBox->button(QDialogButtonBox::Ok)->setFocus();
 
